open Fwd_control;;
open Gny_latex;;
open Protocol_syntax;;

let rec itern f n = function
    [] -> ()
  | h::t -> f n h; itern f (succ n) t;;

let rec mapn f n = function
    [] -> []
  | h::t -> (f n h)::(mapn f (succ n) t);;

let output_proof_tree=true and output_extra_assumptions=true
and proof_tree_fashion_indent = true;;

let { protocol_name = protocol_name;
      assumptions = assumptions;
      messages = messages;
      goals = goals } =
  Gny_parser.question Gny_lexer.rules_lex
    (Lexing.from_channel stdin);;

let hypotheses = assumptions @ (mapn
  (fun k {destination = d; message = f} ->
  (Printf.sprintf "Step \\#%d" k), Gny.Is_told(d,f)) 1 messages);;

Printf.printf "\\section{Analysis of the %s protocol}
\\subsection{Assumptions}
\\begin{description}
" protocol_name;

List.iter (fun (name,formula) ->
  Printf.printf "  \\item[%s] $\\displaystyle %s$\n"
    name (gny_to_latex formula) ) assumptions;

Printf.printf "\\end{description}

\\subsection{Protocol steps}

\\begin{enumerate}
";;

List.iter (fun {source = s;
                destination = d;
                message = f } ->
  Printf.printf "  \\item ${%s} \\rightarrow {%s}:$ $\\displaystyle %s$\n"
    s d (gny_to_latex f) ) messages;

Printf.printf "\\end{enumerate}

\\subsection{Goals}
\\begin{itemize}
";

itern
(fun k conclusion ->
  let conclusion_latex = gny_to_latex conclusion in
  Printf.printf "  \\item $%s$" conclusion_latex;
  flush(stdout);

  match (fwd hypotheses conclusion) with
    Good pr ->
      if output_proof_tree
      then
(*       	Printf.printf ": ok, see the proof in fig.~\\ref{fig:%s%d}
  \\begin{figure}
    $$
    %s
    $$
    \\caption{The %s protocol: a proof of the goal $%s$.}
    \\label{fig:%s%d}
  \\end{figure}
"  protocol_name k (proof_to_latex "    " pr) protocol_name
  conclusion_latex protocol_name k *)

      (if proof_tree_fashion_indent
       then
         Printf.printf "{\footnotesize %s}" (proof_to_latex2 1 pr) 
       else
       	Printf.printf ": ok, the proof is
    $$
    %s
    $$
" (proof_to_latex "    " pr)
      )
      else
        Printf.printf ": ok.\n"
  | Bad extra -> 
      if output_extra_assumptions
      then
	(
	Printf.printf ": unreached, possible extra assumptions are:

  \\begin{itemize}
";
      List.iter (fun f ->
	Printf.printf "\item $%s$\n" (gny_to_latex f) ) extra;
    Printf.printf "\\end{itemize}"
      )
      else Printf.printf "unreached.\n"
) 1 goals;

Printf.printf "\\end{itemize}

\\end{document}
";;
