%{
open Rules_syntax;; open Gny;;
%}

%token IS_TOLD POSSESSES ONCE_CONVEYED BELIEVES FRESH RECOGNIZABLE
%token LEFT_ARROW RIGHT_ARROW VDASH
%token ENCRYPT_LEFT ENCRYPT_RIGHT PLUS MINUS
%token COMMA PAREN_LEFT PAREN_RIGHT DELIMITER SEMICOLON COLON
%token DESTRUCT CONSTRUCT EQUIVALENCE SLASH EXCLAMATION_POINT STAR
%token HONEST_AND_COMPETENT DOUBLE_RIGHT_ARROW CURVY_RIGHT_ARROW
%token EOF
%token <string> Identifier

%start rules
%type <(Rules_syntax.rule_id * Rules_syntax.rule) list> rules
%type <gny> gny gny_nopkey gny_group


%%
gny_nopkey:
    principal IS_TOLD gny { Is_told($1,$3) }
  | principal POSSESSES gny { Possesses($1,$3) }
  | principal ONCE_CONVEYED gny { Once_conveyed($1, $3) }
  | principal BELIEVES gny { Believes($1, $3) }
  | principal DOUBLE_RIGHT_ARROW gny { Jurisdiction($1, $3) }
  | FRESH gny { Fresh($2) }
  | RECOGNIZABLE gny { Recognizable($2) }
  | principal LEFT_ARROW gny RIGHT_ARROW principal { Is_secret($1, $5, $3) }
  | VDASH PLUS pkey_id RIGHT_ARROW principal { Is_public_key($5, $3) }
  | ENCRYPT_LEFT gny_group ENCRYPT_RIGHT PLUS pkey_id { Encrypt_pub($2, $5) }
  | ENCRYPT_LEFT gny_group ENCRYPT_RIGHT MINUS pkey_id { Encrypt_priv($2, $5) }
  | ENCRYPT_LEFT gny_group ENCRYPT_RIGHT gny_nopkey { Encrypt_s($2, $4) }
  | ENCRYPT_LEFT gny_group ENCRYPT_RIGHT EXCLAMATION_POINT gny_nopkey
      { Decrypt_s($2, $5) }
  | function_id PAREN_LEFT gny PAREN_RIGHT { Unary_func($1, $3) }
  | function_id PAREN_LEFT gny COMMA gny PAREN_RIGHT {Binary_func($1, $3, $5)}
  | variable_id { Variable($1) }
  | STAR gny_nopkey { Not_originated_here($2) }
  | PAREN_LEFT gny_group PAREN_RIGHT { $2 }
  | principal HONEST_AND_COMPETENT { Honest_and_competent($1) }
  | PAREN_LEFT gny CURVY_RIGHT_ARROW gny PAREN_RIGHT
      { Leading_to($2, $4) }

gny:
    gny_nopkey { $1 }
  | PLUS pkey_id { Public_key($2) }
  | MINUS pkey_id { Private_key($2) }
  
gny_group:
    gny { $1 }
  | gny COMMA gny_group { Pair($1, $3) }

formula:
    gny { $1 }

formulae:
    formula { [$1] }
  | formula SEMICOLON formulae { $1::$3 }

rule_val:
    CONSTRUCT formulae VDASH formulae
      { Construct($2, $4) }
  | DESTRUCT formulae SLASH formulae VDASH formulae
      { Destruct($2, $4, $6) }
  | DESTRUCT formulae VDASH formulae
      { Destruct($2, [], $4) }
  | EQUIVALENCE formula VDASH formula
      { Equivalence($2, $4) }

rule:
    rule_id COLON rule_val DELIMITER { $1, $3 }

rules:
    EOF { [] }
  | rule rules { $1::$2 }

principal:
    Identifier { $1 }

pkey_id:
    Identifier { $1 }

function_id:
    Identifier { $1 }

variable_id:
    Identifier { $1 }

rule_id:
    Identifier { $1 }

